%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% https://www.iriscores.com
% Author: Vojtech Ters 2016/2022
% vojta.ters@iriscores.com
% @brief: Calculates the Surface of the Elliptical PLane
% a            ... Satellite's Major Axis [m]
% e            ... Satellite's Eccentricity [-]
% Ascension    ... Satellite's Ascension [rad]
% Perilapsis   ... Satellite's Perilapsis [rad]
% Inclination  ... Satellite's Inclination [rad]
% OrbitTime    ... Orbit period of the satellite [s]
% NT           ... Amount of samples per satellite (Smoothness of the elipse)
% ShowWB       ... 1-> Ask for waitbar to show the progress
% NOTE: Output can be fed to "trisurf()"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ TriOut, XOut, YOut, ZOut ] = Calc_Elliptical_Surface( a, e, Ascension, Perilapsis, Inclination, OrbitTime, NT,ShowWB )
 
 N = length(a);
 
 
 %%%%%%%%%%%%%%%%%%%%%
 % Initialize Output %
 TriOut = cell(N,1);
 XOut   = cell(N,1);
 YOut   = cell(N,1);
 ZOut   = cell(N,1);
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare ProgressBar for Canceling %
if(isequal(ShowWB,1))
    WaitB = waitbar(0,'1','Name','Computing Keplerian Elipse ...','CreateCancelBtn',...
                'setappdata(gcbf,''canceling'',1)');
    setappdata(WaitB,'canceling',0);
end

 try
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     % Loop Across All Satellites %
     for i = 1:N
         
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Check if User has Canceled the Request %
        if(isequal(ShowWB,1))
            if getappdata(WaitB,'canceling')
                break;
            end
        end

         % EXTRACT CURRENT PARAMETERS
         Sat_a    = a(i);
         Sat_e    = e(i);
         Inc      = Inclination(i);
         Peri     = Perilapsis(i);
         Asc      = Ascension(i);
         OrbTime  = OrbitTime(i);

         t = linspace(0,OrbTime,NT);
         XS = zeros(1,NT);
         YS = zeros(1,NT);
         ZS = zeros(1,NT);

         %%%%%%%%%%%%%%%%%%%%%%%%%%%
         % Loop Across Orbit Times %
         for j = 1:NT
            Time = t(j);                                      % Extract current Time [s]
            M = 2*pi*(Time)/OrbTime;                          % Mean Anomaly [rad]
            E = KeplerSolve(M,Sat_e,1e-4);                    % Eccentric anomaly [rad]
            v = 2*atan(sqrt((1+Sat_e)/(1-Sat_e))*tan(E/2));   % True Anomaly [rad]
            r = Sat_a*(1-Sat_e^2)/(1+Sat_e * cos(v));         % Radius from center [m]


            %%%%%%%%%%%%%%%%%%%%%%%%%
            % Calculate Coordinates %
            XS(j) = r*( cos(Asc)*cos(Peri + v) - sin(Asc)*sin(Peri + v)*cos(Inc) );
            YS(j) = r*( sin(Asc)*cos(Peri + v) + cos(Asc)*sin(Peri + v)*cos(Inc) );
            ZS(j) = r*( sin(Inc)*sin(Peri + v));  
         end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % Calculate and Save Delaunay Triang. and Position %
         TriOut{i} = delaunay(XS,YS);
         XOut{i} = XS;
         YOut{i} = YS;
         ZOut{i} = ZS;
         
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         % Report Current Estimate of the Remaining Work %
         if(isequal(ShowWB,1))
            waitbar(i/N,WaitB,['Satellite: ',num2str(i),'/',num2str(N)]);
         end

     end
 catch ME
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     % Safely Delete the Waitbar and Rethrow the Error %
     if(isequal(ShowWB,1))
        delete(WaitB);
     end
     rethrow(ME);
 end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If there was no error,delete the waitbar as Hell %
if(isequal(ShowWB,1))
    delete(WaitB);
end


end

