%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% https://www.iriscores.com
% Author: Vojtech Ters 2016/2022
% vojta.ters@iriscores.com
% @Brief: Updates the Satellite Table
%
% Each Satellite has its own Row
% Column Indexeses:
% 1 - Speed v
% 2 - Eccentricity e
% 3 - Major Axis a
% 4 - Minor Axis b
% 5 - Perilapsis [deg]
% 6 - Ascending [deg]
% 7 - inclination [deg]
% 8 - Show Elipse in the 3D [logic]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Callback_Satellite_Table( Src,Event )

global S;

Index_Speed         = 1;
Index_Eccentricity  = 2;
Index_Major_Axis    = 3;
Index_Minor_Axis    = 4;
Index_Perilapsis    = 5;
Index_Ascending     = 6; 
Index_Inclination   = 7;
Index_Ellipse       = 8;

if(isempty(Event.Error))

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Grab 2D Source Index Event from the Table %
    ColumnIndex = Event.Indices(2);
    RowIndex    = Event.Indices(1);
    CellData    = Event.NewData;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % NOTE: Row index correspond to a satellite
    % NOTE: Column Index as Described above
    switch ColumnIndex

        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Major_Axis % -------------------------------------------------------------------------------------------------------------
            S.a(RowIndex) = S.BaseDist*CellData;                                                    % Update Major Axis
            S.e(RowIndex) = sqrt(1 - (S.b(RowIndex)^2)./(S.a(RowIndex).^2));                        % Update Eccentricity
            S.OrbitTime(RowIndex) = 2*pi*sqrt(max(S.a(RowIndex),S.b(RowIndex))^3./(S.Kappa*S.Mz));  % Update Orbit Time
            S.SatTable.Data{RowIndex,2} = S.e(RowIndex);                                            % Update the value in the Table 
            
        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Minor_Axis % -------------------------------------------------------------------------------------------------------------
            S.b(RowIndex) = S.BaseDist*CellData;                                                    % Update Minor Axis
            S.e(RowIndex) = sqrt(1 - (S.b(RowIndex)^2)./(S.a(RowIndex).^2));                        % Update Eccentricity
            S.OrbitTime(RowIndex) = 2*pi*sqrt(max(S.a(RowIndex),S.b(RowIndex))^3./(S.Kappa*S.Mz));  % Update Orbit Time
            S.SatTable.Data{RowIndex,2} = S.e(RowIndex);                                            % Update the value in the Table 
        
        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Perilapsis % -------------------------------------------------------------------------------------------------------------
            S.Perilapsis(RowIndex) = deg2rad(CellData);
        
        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Ascending % --------------------------------------------------------------------------------------------------------------
            S.Ascending(RowIndex) = deg2rad(CellData);

        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Inclination % ------------------------------------------------------------------------------------------------------------
            S.Inclination(RowIndex) = deg2rad(CellData); 

        % -----------------------------------------------------------------------------------------------------------------------------------
        case Index_Ellipse % ----------------------------------------------------------------------------------------------------------------
          % No Action Required
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Ellipse in case it is Selected and Parameters have Changed       %
    % Note: Velocity and Eccentricity are updated automatically. Do not Edit. %
    if(ColumnIndex > 2 )
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Remove the Elliplical Surface %
        DeleteEllipse(RowIndex);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Recalculate the Surface %
        if (S.SatTable.Data{RowIndex,8})
          CreateEllipse(RowIndex);
        end
        
        %%%%%%%%%%%%%%%%%%%%%
        % Reset Axis Limits %
        ResetLimits();
    end
    
    %%%%%%%%%%
    % Redraw %
    drawnow;
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @Brief: Removes the Elliptical Surface from the 3D Space
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DeleteEllipse(SatID)
    global S;
    if(~isempty(S.SatEllipse{SatID}))
        if(isvalid(S.SatEllipse{SatID}))
            delete (S.SatEllipse{SatID});
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @Brief: Computes and Displays Elliptical Surface for the Given Satellite 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function CreateEllipse(SatID)
    global S;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Recalculate the Surface %
    [Tri,X,Y,Z] = Calc_Elliptical_Surface( S.a(SatID), S.e(SatID), S.Ascending(SatID),S.Perilapsis(SatID), S.Inclination(SatID), S.OrbitTime(SatID), S.NumEllipTiles, 0 );
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % Display the Surface %
    S.SatEllipse{SatID} = trisurf(Tri{1},X{1},Y{1},Z{1});
    S.SatEllipse{SatID}.FaceColor = S.Cls(S.Cl_Green,:);    % Fixed Color
    %S.SatEllipse{SatID}.FaceColor = S.ColorEllip(SatID,:);  % Custom Colors
    S.SatEllipse{SatID}.FaceAlpha = S.FaceAlpha;
    S.SatEllipse{SatID}.EdgeColor = 'none';
      
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @Brief: Reset Global Axis Limits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ResetLimits()
    global S;
    S.MinX = -S.BaseDist;
    S.MaxX =  S.BaseDist;
    S.MinY = -S.BaseDist;
    S.MaxY =  S.BaseDist;
    S.MinZ = -S.BaseDist;
    S.MaxZ =  S.BaseDist;
    S.VisAx.XLim = [S.MinX,S.MaxX];
    S.VisAx.YLim = [S.MinY,S.MaxY];
    S.VisAx.ZLim = [S.MinZ,S.MaxZ];
    
    %%%%%%%%%%%%%%%%%%%
    % Reset Plot Data %
    for i = 1:S.NumSat
        for j = 1:S.MaxPoints
            S.PlotOrb{i,j}.XData = 0;
            S.PlotOrb{i,j}.YData = 0;
            S.PlotOrb{i,j}.ZData = 0;
        end
    end

end

