close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Kepler Orbital Movement Demo 3D V1.1
% More info at: https://en.wikipedia.org/wiki/Orbital_mechanics
% https://www.iriscores.com
% Author: Vojtech Ters 2016/2022
% vojta.ters@iriscores.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% #1
% Velocity is not accurate,as it is  calculated as the total distance
% between 2 points in 3D Space. Therefore it is dependent on the
% discretization in time.

% #2
% Actually the real time is not a multiple of TimeScale*TimerPeriod.
% This is not accurate for small periods when the timer is mostly  busy
% drawing and calculating. The real simulation time can be obtained by 
% measuring the timer period and then multiplying it with the TimeScale.

% #3
% The True Anomaly is not displayed in the table,I found no real use for
% it. Code can be however slightly changed to display this value as well.

% #4
% In case of slow Simulation, decrease amount of satellites and or the
% Maximum amount of visible points. Recommended to run on Intel i3/5/i7.
% The parallel (parfor) implementation proved to be extremely slow.

% #5
% The window Size can be changed in here in the main script. Resizing when
% the GUI is running is not allowed however, as it doesnt refrect the changes in
% JTimeSlider.



global S;

S.NumSat = 3;                                              % Amount of Satellites
S.Kappa = 6.67408e-11;                                     % Gravitation Constant
S.Mz = 5.97219e24;                                         % Earth's mass [kg] 
S.Rz = 6378e3;                                             % Earth Radius [m]
S.Alt = 20200e3;                                           % Orbiting Altitude [m]
S.BaseDist = S.Rz + S.Alt;                                 % Base Distance [m]
S.a = S.BaseDist*ones(1,S.NumSat).*linspace(1,2,S.NumSat); % Satellites's main elip. axis [m]
S.b = S.BaseDist*ones(1,S.NumSat);                         % Satellite's sec. elip. axis [m]
S.Ascending     = deg2rad(360*rand(1,S.NumSat));           % Satellite's Ascending node [rad]
S.Inclination   = deg2rad(90*rand(1,S.NumSat));            % Satellite's Inclination [rad]
S.Perilapsis    = deg2rad(90*rand(1,S.NumSat));            % Satellite's arg. of Perilapsis [rad]
S.ElapsedTime   = 0;                                       % Calculated by timer [s]
S.TimeScale     = 600;                                     % Amount of time to increase per timer tick [s]
S.TimerPeriod   = 0.001;                                   % Timer Period [s]
S.MaxPoints     = 50;                                      % Maximum amount of points per satellite [-]
S.NumEllipTiles = 150;                                     % Precision of the eventually plotted ellipse [-]
S.FaceAlpha     = 0.5;                                     % Opacity of the visible surfaces [-] {0 -> 1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Eccentricity and Init Variables %
S.e = sqrt(1 - (S.b.^2)./(S.a.^2));              % [-]
S.OrbitTime = 2*pi*sqrt(S.a.^3./(S.Kappa*S.Mz)); % [s] 
S.DataX = zeros(S.NumSat,S.MaxPoints);
S.DataY = zeros(S.NumSat,S.MaxPoints);
S.DataZ = zeros(S.NumSat,S.MaxPoints);
S.XMin = -S.BaseDist;
S.XMax =  S.BaseDist;
S.YMin = -S.BaseDist;
S.YMax =  S.BaseDist;
S.ZMin = -S.BaseDist;
S.ZMax =  S.BaseDist;
S.SatEllipse = cell(S.NumSat,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %
S.Displ = groot();
S.ScWidth = S.Displ.ScreenSize(3);
S.ScHeight = S.Displ.ScreenSize(4);
S.FigWidth = 1440;   % Figure Width
S.FigHeight = 920;   % Figure Height

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define Colors and Point Properties %
PointMaxSize = 22; % Biggest Paint Size
PointMinSize = 1;  % Smallest Point Size
S.ColorData = morgenstemning(S.MaxPoints);  % Colors of the moving points
S.ColorEllip = jet(S.NumSat);               % Colors of the eliptical surfaces
S.PointSize = linspace(PointMaxSize,PointMinSize,S.MaxPoints); 

%%%%%%%%%%%%%%%%%%%%%%%%%
% Define Generic Colors %
S.Cls = perms([0,102,204])./255;
S.Cl_Brown      = 1;
S.Cl_Dark_Red   = 2;
S.Cl_Dark_Green = 3;
S.Cl_Violet     = 4;
S.Cl_Green      = 5;
S.Cl_Blue       = 6; 
S.ClPlane       = S.Cl_Blue;

%%%%%%%%%%%%%%%%%%%%%%
% Create Main Figure %
S.Fm = figure('name','Kepler''s Orbital Simulation in 3D V1.1');
S.Fm.Position = [S.ScWidth/2-S.FigWidth/2,S.ScHeight/2-S.FigHeight/2,S.FigWidth,S.FigHeight];
S.Fm.NumberTitle    = 'off';
S.Fm.Color          = 'White';
S.Fm.Tag            = 'MainFigure';
S.Fm.MenuBar        = 'none';
S.Fm.ToolBar        = 'figure';
S.Fm.Resize         = 'off';
S.Fm.CloseRequestFcn = @Callback_Figure_Close;

%%%%%%%%%%%%%%%%%%%%%%
% Update Figure Icon %
warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
javaFrame = get(S.Fm,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon(['Images',filesep,'Earth.png']));
clear javaFrame;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove Unnecessary Components from ToolBar %
delete(findall(S.Fm,'ToolTipString','Open File'));
delete(findall(S.Fm,'ToolTipString','Print Figure'));
delete(findall(S.Fm,'ToolTipString','New Figure'));
delete(findall(S.Fm,'ToolTipString','Hide Plot Tools'));
delete(findall(S.Fm,'ToolTipString','Show Plot Tools and Dock Figure'));
delete(findall(S.Fm,'ToolTipString','Link Plot'));
delete(findall(S.Fm,'ToolTipString','Brush/Select Data'));
delete(findall(S.Fm,'ToolTipString','Edit Plot'));
delete(findall(S.Fm,'ToolTipString','Insert Legend'));
delete(findall(S.Fm,'ToolTipString','Insert ColorBar'));
Tb = findall(S.Fm,'tag','FigureToolBar');
Ty = findall(Tb,'Type','uitoggletool');
set(Ty,'Separator','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define User Interface Spacing %
S.AxSpacing         = 0.15;
S.Spacing           = 0.05;
S.BGSpacing         = 0.01;
S.ButtonGroupWidth  = 0.25;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add Visualization Panels & Chart %
DifTime = 5;
S.MainPanel = uipanel(S.Fm,'Tag','Main Panel');
S.UiPanTimer = uipanel(S.MainPanel,'Tag','TimerPanel','Title','Timer & Config');
S.UiPanTimer.Position = [S.BGSpacing,S.BGSpacing,(1-3*S.BGSpacing)/DifTime,1-2*S.BGSpacing];
S.UiPanVisual = uipanel(S.MainPanel,'Tag','Visualization Panel','Title','Visualization');
S.UiPanVisual.Position = [ (2*S.BGSpacing +(1-3*S.BGSpacing)/DifTime), (S.BGSpacing), ((DifTime - 1)*(1-3*S.BGSpacing)/DifTime), (1-2*S.BGSpacing)];


S.VisAx = axes('parent',S.UiPanVisual);
S.PlotOrb = cell(S.NumSat,S.MaxPoints);

%%%%%%%%%%%%%%%%%%%%%%%
% Create Plot Handles %
for i = 1: S.NumSat
    S.PlotOrb{i,1} = plot(0,0,'.','markersize',S.PointSize(1),'parent',S.VisAx,'MarkerFaceColor',S.ColorData(i,:),'color',S.ColorData(i,:));
    hold on;grid on;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Additional Plot Handles %
for i = 1:S.NumSat
    for j = 2:S.MaxPoints
        S.PlotOrb{i,j} = plot(0,0,'.','markersize',S.PointSize(j),'parent',S.VisAx,'MarkerFaceColor',S.ColorData(j,:),'color',S.ColorData(j,:));
        hold on;grid on;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add Earth to the Center %
EarthPrecision = 1000;
[Xz,Yz,Zz] = sphere(EarthPrecision);
Xz = S.Rz*Xz;Yz = Yz * S.Rz;Zz = Zz * S.Rz;
S.TexRawE = im2double(imread(['Images',filesep,'EarthTex.jpg']));
S.TexE = flipud(imresize(S.TexRawE,size(Xz)));
S.SurfE = surf(Xz,Yz,Zz,S.TexE,'FaceColor','texturemap','edgecolor','none','parent',S.VisAx);


GrayC = 1.0;
S.VisAx.XLabel.String = 'X';
S.VisAx.YLabel.String = 'Y';
S.VisAx.ZLabel.String = 'Z';
S.VisAx.XTick = [];
S.VisAx.YTick = [];
S.VisAx.ZTick = [];
S.VisAx.Clipping = 'off';
S.VisAx.BoxStyle = 'full';
S.VisAx.Color = GrayC*ones(1,3);                 % Axes Background Color
S.UiPanVisual.BackgroundColor = GrayC*ones(1,3); % Panel BackgroundColor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add Buttons And Edit Boxes %
S.NumControls = 8; VertC = 1; 
S.ContWidth = (1 - 2*S.Spacing); 
S.ContHeigh = (1 - (S.NumControls+1)*S.Spacing)/S.NumControls;                                                                                              % LEFT      BOTTOM                                WIDTH           HEIGHT
S.ButtonStart = uicontrol('style','pushbutton','parent',S.UiPanTimer,'string','Start','units','normalized','position',                                  [S.Spacing, 1-S.ContHeigh*VertC - S.BGSpacing*VertC, 1-S.Spacing, S.ContHeigh]);VertC = VertC + 1; % StartButton
S.ButtonStop =  uicontrol('style','pushbutton','parent',S.UiPanTimer,'string','Stop','units','normalized','position',                                   [S.Spacing, 1-S.ContHeigh*VertC - S.BGSpacing*VertC, 1-S.Spacing, S.ContHeigh]);VertC = VertC + 1; % StopButton
S.TimeSlLabel = uicontrol('style','text','parent',S.UiPanTimer,'string','Time Scale: ','FontSize',10,'units','normalized','position',                   [S.Spacing, (1 - S.ContHeigh*VertC - S.BGSpacing*VertC + S.ContHeigh/2), (1-S.Spacing*3)/2, S.ContHeigh/2]); % Label
S.TimeSlEdit = uicontrol('style','edit','parent',S.UiPanTimer,'string',num2str(S.TimeScale),'units','normalized','position',                            [2*S.Spacing + (1-3*S.Spacing)/2,   (1 - S.ContHeigh*VertC  - S.BGSpacing*VertC + S.ContHeigh/2), (1-S.Spacing*3)/2,S.ContHeigh/2]);VertC  = VertC + 0.5; % EditBox
S.TimeSlider = uicontrol('style','slider','visible','off','parent',S.UiPanTimer,'string','Adjust Time Speed','units','normalized','position',           [S.Spacing,   (1 - S.ContHeigh*VertC  - S.BGSpacing*VertC),S.ContWidth,S.ContHeigh]); % Standard Slider
%%% TimeSlider is not really used in the code,its purpose is to obtain pixel coordinates for the JSlider.

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Satellite Table %
UIS = 0.050; % TABLE HEIGH
S.SatTable = uitable('parent',S.UiPanTimer,'units','normalized');
S.SatTable.ColumnName = {'v [m/s]','e','a','b','<html>&omega<html>','<html>&Omega<html>','<html>&iota<html>','Ellip.'};
S.SatTable.ColumnEditable = [false,false,true,true,true,true,true,true];
S.SatTable.ColumnFormat = {'numeric','numeric','numeric','numeric','numeric','numeric','numeric','logical'};
CW = 45;S.SatTable.ColumnWidth = {CW,CW,CW,CW,CW,CW,CW,CW}; % Column Width in Pixels
S.SatTable.Interruptible = 'on';
S.SatTable.Position = [S.Spacing,   (1 - S.ContHeigh*VertC - S.BGSpacing*(VertC + 1) - S.ContHeigh/2*S.NumSat - UIS) ,   S.ContWidth,  (S.ContHeigh/2*S.NumSat + UIS)];
for i = 1:S.NumSat
    S.SatTable.Data{i,1} = 0;
    S.SatTable.Data{i,2} = S.e(i);                % Normalized 
    S.SatTable.Data{i,3} = S.a(i)/S.BaseDist;     % Normalized to BaseDist
    S.SatTable.Data{i,4} = S.b(i)/S.BaseDist;     % Normalized to BaseDist
    S.SatTable.Data{i,5} = rad2deg(S.Perilapsis(i));
    S.SatTable.Data{i,6} = rad2deg(S.Ascending(i));
    S.SatTable.Data{i,7} = rad2deg(S.Inclination(i));
    S.SatTable.Data{i,8} = logical(false);
end
PrevPosition = S.SatTable.Position;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Equatorial Plane Checkbox %
S.CheckEqPlane = uicontrol('style','checkbox','parent',S.UiPanTimer,'string','Show Equatorial Plane','units','normalized','position',[S.Spacing,PrevPosition(2) - S.BGSpacing - S.ContHeigh,1-S.Spacing,S.ContHeigh]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CREATE UNDOCUMENTED JSLIDER %
S.JTimeSlider = javax.swing.JSlider;
set(S.JTimeSlider,'minimum',1,'maximum',5000,'Value',S.TimeScale,'MinorTickSpacing',1000, 'MajorTickSpacing',4999 , 'PaintLabels',true);
javacomponent(S.JTimeSlider,getpixelposition(S.TimeSlider,true));

%%%%%%%%%%%%%%%%%%%
% Setup CallBacks %
S.ButtonStart.Callback      = @Callback_Timer_Start;
S.ButtonStop.Callback       = @Callback_Timer_Stop;
S.TimeSlider.Callback       = @Callback_Slider;
S.TimeSlEdit.Callback       = @Callback_TimeScale;
S.SatTable.CellEditCallback = @Callback_Satellite_Table;
S.CheckEqPlane.Callback     = @Callback_Equatorial_Plane;
HandleToJSlider = handle(S.JTimeSlider, 'CallbackProperties');
HandleToJSlider.StateChangedCallback = @Callback_Slider;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Simulation Timer %
S.Tm = timer('BusyMode','drop','ExecutionMode','fixedSpacing');
S.Tm.Period   = S.TimerPeriod;
S.Tm.StartFcn = @Callback_Timer_Start;
S.Tm.StopFcn  = @Callback_Timer_Stop;
S.Tm.TimerFcn = @TimerFunction;


