%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief: Updates Position of all Satellites by taking into account actual Time 
% https://www.iriscores.com
% Author: Vojtech Ters 2016/2022
% vojta.ters@iriscores.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function TimerFunction( src,Event )

global S;

AxChange = 0;
if(strcmp(S.Tm.Running,'on'))
    
    %%%%%%%%%%%%%%%%%%%%%%%%
    % Update Absolute Time % 
    S.ElapsedTime = S.ElapsedTime + S.TimeScale;


    %%%%%%%%%%%%%%%%%%
    % Shift Data --> %
    S.DataX = circshift(S.DataX,[0,1]);
    S.DataY = circshift(S.DataY,[0,1]);
    S.DataZ = circshift(S.DataZ,[0,1]); 
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Loop across All Satellites %
    for i = 1:S.NumSat
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Extract Current Satellite Parameters %
        Sat_a           = S.a(i);
        Sat_e           = S.e(i);
        SatOrbitTime    = S.OrbitTime(i);    % [s]
        SatInclination  = S.Inclination(i);  % [rad]
        SatPerilapsis   = S.Perilapsis(i);   % [rad]
        SatAscending    = S.Ascending(i);    % [rad]
        
        %%%%%%%%%%%%%%%%%%%%%%%%%
        % Solve Kepler Equation %
        M = 2*pi*(S.ElapsedTime)/SatOrbitTime;                          % Mean Anomaly [rad]
        E = KeplerSolve(M,Sat_e,1e-4);                                  % Eccentric anomaly [rad]
        v = 2*atan(sqrt((1+Sat_e)/(1-Sat_e))*tan(E/2));                 % True Anomaly [rad]
        r = Sat_a*(1-Sat_e^2)/(1+Sat_e * cos(v));                       % Radius from center [m]
        %h = sqrt(S.Kappa*S.Mz*(1 - Sat_e^2));                          % Angular Momentum
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Calculate New Coordinates %
        XData = r*( cos(SatAscending)*cos(SatPerilapsis + v) - sin(SatAscending)*sin(SatPerilapsis + v)*cos(SatInclination) );
        YData = r*( sin(SatAscending)*cos(SatPerilapsis + v) + cos(SatAscending)*sin(SatPerilapsis + v)*cos(SatInclination) );
        ZData = r*( sin(SatInclination)*sin(SatPerilapsis + v));
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%
        % Update Display Limits %
        if( (XData < S.XMin) || (XData > S.XMax) || (YData < S.YMin) || (YData > S.YMax) || (ZData < S.ZMin) || (ZData > S.ZMax))
            AxChange = 1;
            if(XData < S.XMin)
              S.XMin = XData;
            end
            if(XData > S.XMax)
              S.XMax = XData;
            end
            if(YData < S.YMin)
              S.YMin = YData;
            end
            if(YData > S.YMax)
              S.YMax = YData;
            end
            if(ZData < S.ZMin)
              S.ZMin = ZData;
            end
            if(ZData > S.ZMax)
              S.ZMax = ZData;
            end

        end
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Save Position for Display %
        S.DataX(i,1) = XData;
        S.DataY(i,1) = YData;
        S.DataZ(i,1) = ZData;

        %%%%%%%%%%%%%%%%%%%%%%
        % Update Plot's Data %
        for j = 1:S.MaxPoints
            S.PlotOrb{i,j}.XData = S.DataX(i,j);
            S.PlotOrb{i,j}.YData = S.DataY(i,j);
            S.PlotOrb{i,j}.ZData = S.DataZ(i,j);
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Calculate and Update Velocity %
        Vec1 = [S.DataX(i,1),S.DataY(i,1),S.DataZ(i,1)];
        Vec2 = [S.DataX(i,2),S.DataY(i,2),S.DataZ(i,2)];
        Dist = sqrt(sum((Vec1-Vec2).^2));
        S.SatTable.Data{i,1} = round(Dist/S.TimeScale);

    end
   
    %%%%%%%%%%%%%%%%%%%%%%
    % Update Axis Limits %
    MI = min([S.XMin,S.YMin,S.ZMin]);
    MA = max([S.XMax,S.YMax,S.ZMax]);
    xlim([MI,MA]);
    ylim([MI,MA]);
    zlim([MI,MA]);

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Equatorial Surface %
    if(isequal(AxChange,1) && isequal(S.CheckEqPlane.Value,1))
        Callback_Equatorial_Plane(0,0);
    end
    
    %%%%%%%%%%
    % Redraw %
    drawnow;

end
end

